package exnihilocreatio.blocks;

import exnihilocreatio.ModBlocks;
import exnihilocreatio.compatibility.ITOPInfoProvider;
import exnihilocreatio.config.ModConfig;
import exnihilocreatio.items.tools.ICrook;
import exnihilocreatio.tiles.BaseTileEntity;
import exnihilocreatio.tiles.ITileLeafBlock;
import exnihilocreatio.tiles.TileInfestingLeaves;
import exnihilocreatio.util.Data;
import exnihilocreatio.util.IHasModel;
import exnihilocreatio.util.LogUtil;
import exnihilocreatio.util.Util;
import mcjty.theoneprobe.api.IProbeHitData;
import mcjty.theoneprobe.api.IProbeInfo;
import mcjty.theoneprobe.api.ProbeMode;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.BlockPlanks.EnumType;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.property.ExtendedBlockState;
import net.minecraftforge.common.property.IExtendedBlockState;
import net.minecraftforge.common.property.IUnlistedProperty;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class BlockInfestingLeaves extends BlockLeaves implements ITileEntityProvider, ITOPInfoProvider, IHasModel {

    public static final PropertyBool NEARBYLEAVES = PropertyBool.func_177716_a("nearby_leaves");

    public static final IUnlistedProperty<IBlockState> LEAFBLOCK = new IUnlistedProperty<IBlockState>() {
        @Override
        public String getName() {
            return "LeafBlock";
        }

        @Override
        public boolean isValid(IBlockState value) {
            return true;
        }

        @Override
        public Class<IBlockState> getType() {
            return IBlockState.class;
        }

        @Override
        public String valueToString(IBlockState value) {
            return value.toString();
        }
    };

    public BlockInfestingLeaves() {
        this(InfestedType.INFESTING);
        this.func_149663_c("block_infesting_leaves");
        this.setRegistryName("block_infesting_leaves");
        Data.BLOCKS.add(this);
        this.func_180632_j(
                this.field_176227_L.func_177621_b().func_177226_a(field_176236_b, false).func_177226_a(field_176237_a, false).func_177226_a(NEARBYLEAVES, true));
    }

    protected BlockInfestingLeaves(InfestedType type) {
        super();
        this.func_149675_a(true);
        this.field_185686_c = true;
    }

    /**
     * Sets a normal leave to infested
     */
    public static void infestLeafBlock(World world, IBlockState state, BlockPos pos) {
        IBlockState leafState;
        //Prevents a crash with forestry using the new model system
        if (Block.field_149771_c.func_177774_c(state.func_177230_c()).func_110624_b().equalsIgnoreCase("forestry"))
            leafState = Blocks.field_150362_t.func_176223_P();
        else leafState = state;
        world.func_180501_a(pos, ModBlocks.infestingLeaves.func_176223_P().func_177226_a(field_176237_a, true), 3);
        if (world.func_175625_s(pos) != null)
            ((ITileLeafBlock) world.func_175625_s(pos)).setLeafBlock(leafState);
    }

    /**
     * Used to set an existing infesting leaves block to infested
     *
     * @param world     The world this is happening in
     * @param pos       The position of the block
     * @param leafState The leaf state captured by the infesting leaves block block
     */
    public static void setInfested(World world, BlockPos pos, IBlockState leafState) {
        IBlockState block = world.func_180495_p(pos);
        if (block.func_177230_c() instanceof BlockInfestingLeaves) {
            IBlockState retval = ((IExtendedBlockState) ModBlocks.infestedLeaves.func_176223_P())
                    .withProperty(BlockInfestedLeaves.LEAFBLOCK, leafState)
                    .func_177226_a(field_176236_b, true)
                    .func_177226_a(field_176237_a, false);

            world.func_180501_a(pos, retval, 0b111);

            if (world.func_175625_s(pos) != null) {
                ((ITileLeafBlock) world.func_175625_s(pos)).setLeafBlock(leafState);
                ((BaseTileEntity) world.func_175625_s(pos)).markDirtyClient();
            }
        } else if (Util.isLeaves(block) && !(block.func_177230_c() instanceof BlockInfestedLeaves)) {
            LogUtil.error("Sent leaf change to wrong method, redirecting");
            infestLeafBlock(world, block, pos);
        }
    }

    public static void spread(World world, BlockPos pos, IBlockState state, Random rand) {
        if (!world.field_72995_K && state != null) {
            if (state.func_177229_b(NEARBYLEAVES)) {
                NonNullList<BlockPos> nearbyLeaves = Util.getNearbyLeaves(world, pos);
                if (nearbyLeaves.isEmpty())
                    world.func_180501_a(pos, state.func_177226_a(NEARBYLEAVES, false), 7);
                else {
                    TileEntity te = world.func_175625_s(pos);
                    if (te instanceof TileInfestingLeaves && ((TileInfestingLeaves) te).getProgress() > ModConfig.infested_leaves.leavesSpreadPercent) {
                        nearbyLeaves.stream().filter(leaves -> rand.nextFloat() <= ModConfig.infested_leaves.leavesSpreadChanceFloat).findAny().ifPresent(blockPos -> BlockInfestingLeaves.infestLeafBlock(world, world.func_180495_p(blockPos), blockPos));
                    }
                }
            }
        }
    }

    public static IBlockState getLeafState(IBlockState state) {
        if (state instanceof IExtendedBlockState) {
            return ((IExtendedBlockState) state).getValue(BlockInfestedLeaves.LEAFBLOCK);
        }

        return state;
    }

    @Override
    public void func_180645_a(@Nonnull World world, @Nonnull BlockPos pos, @Nonnull IBlockState state, @Nonnull Random rand) {
        this.func_180650_b(world, pos, state, rand);
        spread(world, pos, state, rand);
    }

    @Override
    public int func_149738_a(World worldIn) {
        return ModConfig.infested_leaves.leavesUpdateFrequency;
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos) {
        if (!worldIn.field_72995_K && Util.isLeaves(worldIn.func_180495_p(fromPos)))
            worldIn.func_180501_a(pos, state.func_177226_a(NEARBYLEAVES, true), 7);
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
    }

    @Override
    @Nonnull
    @Deprecated
    public EnumBlockRenderType func_149645_b(IBlockState state) {
        return EnumBlockRenderType.ENTITYBLOCK_ANIMATED;
    }

    @Override
    @Nonnull
    public IBlockState getExtendedState(@Nonnull IBlockState state, IBlockAccess world, BlockPos pos) {
        if (state instanceof IExtendedBlockState) {
            IExtendedBlockState retval = (IExtendedBlockState) state;
            IBlockState leafState;
            TileEntity te = world.func_175625_s(pos);

            if (te instanceof ITileLeafBlock) {
                leafState = ((ITileLeafBlock) te).getLeafBlock();
            } else {
                leafState = Blocks.field_150362_t.func_176223_P();
            }

            return retval.withProperty(LEAFBLOCK, leafState);
        }
        return state;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e() {
        IProperty<?>[] listedProperties = {field_176236_b, field_176237_a, NEARBYLEAVES};
        IUnlistedProperty<?>[] unlistedProperties = {LEAFBLOCK};
        return new ExtendedBlockState(this, listedProperties, unlistedProperties);
    }

    @Override
    @Nonnull
    @Deprecated
    public IBlockState func_176203_a(int meta) {
        return func_176223_P().func_177226_a(field_176236_b, (meta & 0b100) != 0)
                .func_177226_a(field_176237_a, (meta & 0b010) != 0)
                .func_177226_a(NEARBYLEAVES, (meta & 0b001) != 0);
    }

    /**
     * 3 bits for data:
     * 0               0                0
     * checkDecay      decayable       nearbyLeaves
     */
    @Override
    public int func_176201_c(IBlockState state) {
        int checkDecay = state.func_177229_b(field_176236_b) ? 1 : 0;
        int decayable = state.func_177229_b(field_176237_a) ? 1 : 0;
        int nearbyLeaves = state.func_177229_b(NEARBYLEAVES) ? 1 : 0;

        return checkDecay << 2 | decayable << 1 | nearbyLeaves;
    }

    @Override
    @Nonnull
    public List<ItemStack> getDrops(@Nonnull IBlockAccess world, @Nonnull BlockPos pos, @Nonnull IBlockState state, int fortune) {
        return new ArrayList<>();
    }

    @Override
    @Nonnull
    public List<ItemStack> onSheared(@Nonnull ItemStack item, IBlockAccess world, BlockPos pos, int fortune) {
        ArrayList<ItemStack> ret = new ArrayList<>();
        ret.add(new ItemStack(this));
        return ret;
    }

    @Override
    @Nonnull
    public EnumType func_176233_b(int meta) {
        return EnumType.OAK;
    }

    @Override
    public TileEntity func_149915_a(@Nonnull World worldIn, int meta) {
        return new TileInfestingLeaves();
    }

    @Override
    public void func_176208_a(World world, BlockPos pos, IBlockState state, EntityPlayer player) {
        if (!world.field_72995_K && !player.func_184812_l_()) {
            TileEntity tile = world.func_175625_s(pos);

            if (tile != null) {
                if (tile instanceof TileInfestingLeaves) {
                    TileInfestingLeaves leaves = (TileInfestingLeaves) tile;

                    if (!player.func_184614_ca().func_190926_b()
                            && player.func_184614_ca().func_77973_b() instanceof ICrook) {
                        if (world.field_73012_v.nextFloat() < leaves.getProgress() * ModConfig.crooking.stringChance) {
                            Util.dropItemInWorld(leaves, player, new ItemStack(Items.field_151007_F, 1, 0), 0.02f);
                        }
                    } else if (world.field_73012_v.nextFloat() < leaves.getProgress() * ModConfig.crooking.stringChance / 4.0d) {
                        Util.dropItemInWorld(leaves, player, new ItemStack(Items.field_151007_F, 1, 0), 0.02f);

                    }
                }

                world.func_175713_t(pos);
            }
        }
    }

    @Override
    public void addProbeInfo(ProbeMode mode, IProbeInfo probeInfo, EntityPlayer player, World world,
                             IBlockState blockState, IProbeHitData data) {
        TileInfestingLeaves tile = (TileInfestingLeaves) world.func_175625_s(data.getPos());

        if (tile != null) {
            if (tile.getProgress() >= 100) {
                probeInfo.text("Progress: Done");
            } else {
                probeInfo.progress(tile.getProgress(), 100);
            }
        }

    }

    enum InfestedType {
        INFESTING,
        INFESTED
    }
}
